<?php

namespace Drupal\blazy_ui\Form;

use Drupal\blazy\Blazy;
use Drupal\blazy\BlazyDefault;
use Drupal\blazy\BlazyManagerInterface;

/**
 * Defines blazy admin settings form.
 */
class BlazySettingsForm {

  /**
   * The blazy manager service.
   *
   * @var Drupal\blazy\BlazyManagerInterface
   */
  protected $manager;

  /**
   * Class constructor.
   */
  public function __construct(BlazyManagerInterface $manager) {
    $this->manager = $manager;
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'blazy_settings';
  }

  /**
   * Implements \Drupal\Core\Form\FormInterface::buildForm().
   */
  public function buildForm() {
    $form['admin_css'] = [
      '#type'          => 'checkbox',
      '#title'         => t('Admin CSS'),
      '#default_value' => $this->manager->config('admin_css', TRUE),
      '#description'   => t('Uncheck to disable blazy related admin compact form styling, only if not compatible with your admin theme.'),
    ];

    $form['responsive_image'] = [
      '#type'          => 'checkbox',
      '#title'         => t('Support Picture module'),
      '#default_value' => $this->manager->config('responsive_image', FALSE),
      '#description'   => t('Check to support the <a href="@url">Picture</a> module. Be sure to use blazy-related formatters.', ['@url' => 'https://dgo.to/picture']),
      '#disabled'      => !function_exists('picture_mapping_load'),
    ];

    $form['unbreakpoints'] = [
      '#type'          => 'checkbox',
      '#title'         => t('Disable custom breakpoints'),
      '#default_value' => $this->manager->config('breakpoints', FALSE),
      '#description'   => t('Check to permanently disable custom breakpoints which is always disabled when choosing a Picture. Only reasonable if consistently using Picture.'),
    ];

    $form['one_pixel'] = [
      '#type'          => 'checkbox',
      '#title'         => t('One pixel placeholder'),
      '#default_value' => $this->manager->config('one_pixel', TRUE),
      '#description'   => t('By default a one pixel image is the placeholder for lazyloaded Picture. Useful to perform a lot better. Uncheck to disable, and use Drupal-managed smallest/fallback image style instead. Be sure to add proper dimensions or at least min-height/min-width via CSS accordingly to avoid layout reflow since Aspect ratio is not supported with Picture yet. Disabling this will result in downloading fallback image as well for non-PICTURE element (double downloads).'),
    ];

    $form['blazy'] = [
      '#type'        => 'fieldset',
      '#tree'        => TRUE,
      '#title'       => t('Blazy JS settings'),
      '#description' => t('The following are JS related settings.'),
    ];

    $form['blazy']['loadInvisible'] = [
      '#type'          => 'checkbox',
      '#title'         => t('Load invisible'),
      '#default_value' => $this->manager->config('blazy.loadInvisible', FALSE),
      '#description'   => t('Set to true if you want to load invisible (hidden) elements.'),
    ];

    $form['blazy']['offset'] = [
      '#type'          => 'textfield',
      '#title'         => t('Offset'),
      '#default_value' => $this->manager->config('blazy.offset', 100),
      '#description'   => t("The offset controls how early you want the elements to be loaded before they're visible. Default is <strong>100</strong>, so 100px before an element is visible it'll start loading."),
      '#field_suffix'  => 'px',
      '#maxlength'     => 5,
      '#size'          => 10,
    ];

    $form['blazy']['saveViewportOffsetDelay'] = [
      '#type'          => 'textfield',
      '#title'         => t('Save viewport offset delay'),
      '#default_value' => $this->manager->config('blazy.saveViewportOffsetDelay', 50),
      '#description'   => t('Delay for how often it should call the saveViewportOffset function on resize. Default is <strong>50</strong>ms.'),
      '#field_suffix'  => 'ms',
      '#maxlength'     => 5,
      '#size'          => 10,
    ];

    $form['blazy']['validateDelay'] = [
      '#type'          => 'textfield',
      '#title'         => t('Set validate delay'),
      '#default_value' => $this->manager->config('blazy.validateDelay', 25),
      '#description'   => t('Delay for how often it should call the validate function on scroll/resize. Default is <strong>25</strong>ms.'),
      '#field_suffix'  => 'ms',
      '#maxlength'     => 5,
      '#size'          => 10,
    ];

    $form['visibility'] = [
      '#type'          => 'radios',
      '#title'         => t('Show Blazy on specific pages'),
      '#description'   => t('Blazy uses formatters. However for Blazy HTML filter to work with inline media (specific for D7), we must define pages to load Blazy library here. Only Blazy CSS and JS are loaded where Blazy filter is active. Blazy never overrides images globally.'),
      '#options'       => [0 => t('All pages except those listed'), 1 => t('Only the listed pages')],
      '#default_value' => $this->manager->config('visibility', 0),
    ];

    $form['pages'] = [
      '#type'          => 'textarea',
      '#title'         => '<span class="element-invisible">' . t('Pages') . '</span>',
      '#default_value' => $this->manager->config('pages', Blazy::PAGES),
      '#description'   => t("Specify pages by using their paths. Enter one path per line. The '*' character is a wildcard. Example paths are %blog for the blog page and %blog-wildcard for every personal blog. %front is the front page.", [
        '%blog'          => 'blog',
        '%blog-wildcard' => 'blog/*',
        '%front'         => '<front>',
      ]),
    ];

    $form['#submit'][] = 'blazy_ui_submit_form';

    return system_settings_form($form);
  }

  /**
   * Implements \Drupal\Core\Form\FormInterface::submitForm().
   */
  public function submitForm($form, &$form_state) {
    $defaults = BlazyDefault::formSettings();
    $data = [];

    // Always run typecasting on submit.
    $this->manager->typecast($form_state['values'], 'blazy.settings', TRUE);
    foreach ($defaults as $key => $value) {
      if (isset($form_state['values'][$key])) {
        $data[$key] = $form_state['values'][$key];
      }
    }

    // Merge all separate variables into blazy.settings for simplicity.
    variable_set('blazy.settings', array_merge((array) $this->manager->config(), $data));

    // Safe to remove old array since already merged above.
    foreach ($defaults as $key => $value) {
      if (isset($form_state['values'][$key])) {
        unset($form_state['values'][$key]);
      }
    }
  }

}
