<?php

namespace Drupal\blazy;

/**
 * Defines shared plugin default settings for field formatter and Views style.
 */
class BlazyDefault {

  /**
   * Defines constant for the supported text fields.
   */
  const TEXTS = ['text', 'text_long', 'text_with_summary'];

  /**
   * Defines constant for the supported text tags.
   */
  const TAGS = ['a', 'em', 'strong', 'h2', 'p', 'span', 'ul', 'ol', 'li'];

  /**
   * The supported $breakpoints.
   *
   * @var array
   */
  private static $breakpoints = ['xs', 'sm', 'md', 'lg', 'xl'];

  /**
   * Returns Blazy specific breakpoints.
   */
  public static function getConstantBreakpoints() {
    return self::$breakpoints;
  }

  /**
   * Returns basic plugin settings: text, image, file, entities with grids.
   */
  public static function baseSettings() {
    $settings = [
      'cache'             => 0,
      'current_view_mode' => 'default',
    ] + self::gridSettings();

    $context = ['class' => get_called_class()];
    drupal_alter('blazy_base_settings', $settings, $context);
    return $settings;
  }

  /**
   * Returns optional grid field formatter and Views settings.
   */
  public static function gridSettings() {
    return [
      'grid'        => 0,
      'grid_medium' => 0,
      'grid_small'  => 0,
      'style'       => '',
    ];
  }

  /**
   * Returns image-related field formatter and Views settings.
   */
  public static function baseImageSettings() {
    return [
      'background'             => FALSE,
      'box_caption'            => '',
      'box_caption_custom'     => '',
      'box_style'              => '',
      'box_media_style'        => '',
      'breakpoints'            => [],
      'caption'                => [],
      'image_style'            => '',
      'lazy'                   => 'blazy',
      'media_switch'           => '',
      'ratio'                  => '',
      'responsive_image_style' => '',
      'sizes'                  => '',
    ];
  }

  /**
   * Returns image-related field formatter and Views settings.
   */
  public static function imageSettings() {
    return [
      'thumbnail_style' => '',
      'view_mode'       => '',
    ] + self::baseSettings() + self::baseImageSettings();
  }

  /**
   * Returns Views specific settings.
   */
  public static function viewsSettings() {
    return [
      'class'   => '',
      'id'      => '',
      'image'   => '',
      'link'    => '',
      'overlay' => '',
      'title'   => '',
      'vanilla' => FALSE,
    ];
  }

  /**
   * Returns fieldable entity formatter and Views settings.
   */
  public static function extendedSettings() {
    return self::viewsSettings() + self::imageSettings();
  }

  /**
   * Returns sensible default options common for entities lacking of UI.
   */
  public static function entitySettings() {
    return [
      'blazy'        => TRUE,
      'lazy'         => 'blazy',
      'media_switch' => 'media',
      'ratio'        => 'fluid',
      'rendered'     => FALSE,
      'view_mode'    => 'default',
      '_detached'    => TRUE,
    ];
  }

  /**
   * Returns sensible html settings to shutup notices when lacking.
   */
  public static function itemSettings() {
    return [
      'blazy_data'     => [],
      'content_url'    => '',
      'delta'          => 0,
      'embed_url'      => '',
      'icon'           => '',
      'image_url'      => '',
      'item_id'        => 'blazy',
      'lazy_attribute' => 'src',
      'lazy_class'     => 'b-lazy',
      'lightbox'       => FALSE,
      'picture'        => [],
      'player'         => FALSE,
      'scheme'         => '',
      'type'           => 'image',
      'uri'            => '',
      'use_image'      => FALSE,
      'use_media'      => FALSE,
      'height'         => NULL,
      'width'          => NULL,
    ] + self::imageSettings();
  }

  /**
   * Returns blazy theme properties.
   */
  public static function themeProperties() {
    return [
      'attributes',
      'captions',
      'iframe',
      'image',
      'item',
      'settings',
      'url',
    ];
  }

  /**
   * Returns blazy UI settings for typecasting, done via config schema at D8.
   */
  public static function formSettings() {
    return [
      'admin_css'        => TRUE,
      'responsive_image' => FALSE,
      'unbreakpoints'    => FALSE,
      'one_pixel'        => TRUE,
      'visibility'       => 0,
      'pages'            => Blazy::PAGES,
      'blazy'            => [
        'loadInvisible'           => FALSE,
        'offset'                  => 100,
        'saveViewportOffsetDelay' => 50,
        'validateDelay'           => 25,
      ],
      'filters'          => [
        'column'       => TRUE,
        'grid'         => TRUE,
        'media_switch' => '',
      ],
    ];
  }

}
