<?php

namespace Drupal\blazy;

/**
 * Implements a public facing blazy manager.
 *
 * A few modules re-use this: GridStack, Mason, Slick...
 */
class BlazyManager extends BlazyManagerBase {

  /**
   * Checks for Blazy formatter such as from within a Views style plugin.
   *
   * @var bool
   */
  private $isBlazy = FALSE;

  /**
   * CHecks if the image style contains crop in the effect name.
   *
   * @var array
   */
  private $isCrop;

  /**
   * {@inheritdoc}
   */
  public function config($key = '', $default = NULL, $id = 'blazy.settings', array $defaults = []) {
    return parent::config($key, $default, $id, $id == 'blazy.settings' ? BlazyDefault::formSettings() : $defaults);
  }

  /**
   * Returns the enforced content, or image using theme_blazy().
   *
   * @param array $build
   *   The array containing: item, settings, or optional captions.
   *
   * @return array
   *   The alterable and renderable array of enforced content, or theme_blazy().
   */
  public function getBlazy(array $build = []) {
    if (empty($build['item'])) {
      return [];
    }

    $settings                = &$build['settings'];
    $settings['delta']       = isset($settings['delta']) ? $settings['delta'] : 0;
    $settings['image_style'] = isset($settings['image_style']) ? $settings['image_style'] : '';

    $image = [
      '#theme'       => 'blazy',
      '#image_style' => $settings['image_style'],
      '#item'        => isset($settings['entity_type_id']) && $settings['entity_type_id'] == 'user' ? $build['item'] : [],
      '#build'       => $build,
      '#pre_render'  => ['blazy_pre_render'],
    ];

    drupal_alter('blazy', $image, $settings);
    return $image;
  }

  /**
   * Builds the Blazy as a structured array ready for ::renderer().
   *
   * @param array $element
   *   The pre-rendered element.
   *
   * @return array
   *   The renderable array of pre-rendered element.
   */
  public function preRender(array $element) {
    $build = $element['#build'];
    unset($element['#build']);

    if (empty($build['item'])) {
      return [];
    }

    // Prepare the main image.
    $this->prepareImage($element, $build);

    // Fetch the newly modified settings.
    $settings = $element['#settings'];

    // Provides optional media video if so configured.
    // Does it look familiar, `module_load_include()`, only native?
    if ($settings['use_media'] && empty($settings['_noiframe'])) {
      BlazyMedia::build($element);
    }

    // Provides optional link to content or lightboxes if so configured.
    if ($switch = $settings['media_switch']) {
      if ($switch == 'content' && $settings['content_url']) {
        $element['#url'] = $settings['content_url'];
      }
      elseif ($settings['lightbox']) {
        BlazyLightbox::build($element);
      }
    }

    // We are done, safe to cleanup wasted settings.
    $element['#settings'] = array_filter($element['#settings'], function ($value) {
      return ($value !== NULL && $value !== '' && $value !== 0 && $value !== []);
    });

    return $element;
  }

  /**
   * Prepares the Blazy image as a structured array ready for ::renderer().
   *
   * @param array $element
   *   The renderable array being modified.
   * @param array $build
   *   The array of information containing the required Image or File item
   *   object, settings, optional container attributes.
   */
  protected function prepareImage(array &$element, array $build) {
    $image = $item_attributes = [];
    $attributes = isset($build['attributes']) ? $build['attributes'] : [];
    $item = $build['item'];
    $settings = &$build['settings'];
    $settings += BlazyDefault::itemSettings();
    $picture = !empty($settings['resimage']);
    $settings['ratio'] = $settings['ratio'] ? str_replace(':', '', $settings['ratio']) : FALSE;
    $settings['uri'] = empty($settings['uri']) && isset($item->uri) ? $item->uri : $settings['uri'];

    // Decides if to use image or video, or both.
    $settings['use_media'] = $settings['embed_url'] && in_array($settings['type'], ['audio', 'video']);
    $settings['use_image'] = !empty($settings['uri']);

    // Unlike D8, we have no free $item->_attributes from RDF, provide one here.
    // With or without rdf enabled, no need to check for module_exists().
    if ($settings['type'] == 'image') {
      $item->_attributes['typeof'] = ['foaf:Image'];
    }

    // Extract field item attributes for the theme function, and unset them
    // from the $item so that the field template does not re-render them.
    if (isset($item->_attributes)) {
      $item_attributes = $item->_attributes;
      unset($item->_attributes);
    }

    // Image style modifier can be multi-style images such as GridStack.
    $settings['image_url'] = empty($settings['image_url']) ? file_create_url($settings['uri']) : $settings['image_url'];
    if ($settings['image_style']) {
      $settings['image_url'] = image_style_url($settings['image_style'], $settings['uri']);

      // Only re-calculate dimensions if not cropped, nor already set.
      if (empty($settings['_dimensions'])) {
        $settings = array_merge($settings, Blazy::transformDimensions($settings['image_style'], $item));
      }
    }

    // Aspect ratio to fix layout reflow with lazyloaded images responsively.
    // This is outside 'lazy' to allow non-lazyloaded iframes use this too.
    if ($settings['width'] && !$picture) {
      if ($settings['ratio']) {
        $attributes['class'][] = 'media--ratio media--ratio--' . $settings['ratio'];

        if (in_array($settings['ratio'], ['enforced', 'fluid'])) {
          // If "lucky", Blazy/ Slick Views galleries may already set this once.
          // Lucky when you don't flatten out the Views output earlier.
          // This only makes sense when you have huge AJAX galleries, though.
          $padding = empty($settings['padding_bottom']) ? round((($settings['height'] / $settings['width']) * 100), 2) : $settings['padding_bottom'];
          $attributes['style'] = 'padding-bottom: ' . $padding . '%';
          $settings['_breakpoint_ratio'] = $settings['ratio'];

          // Views rewrite results may strip out `style`, provide hint to JS.
          // This requires a custom JS work since it is an edge case.
          $attributes['data-ratio'] = $padding;
        }
      }

      // Only output dimensions for non-responsive images.
      // Respects hand-coded image attributes.
      if (!isset($item_attributes['width'])) {
        $item_attributes['height'] = $settings['height'];
        $item_attributes['width'] = $settings['width'];
      }
    }

    // The fallback must run as fallback, also for Picture.
    foreach (['width', 'height', 'alt', 'title'] as $key) {
      if (isset($item->{$key})) {
        // Respects hand-coded image attributes, image style, and set once.
        if (array_key_exists($key, $item_attributes)) {
          continue;
        }

        // Do not output an empty 'title' attribute.
        if ($key == 'title' && (strlen($item->title) != 0)) {
          $item_attributes['title'] = $item->title;
        }
        // Ensures to not override dimensions set once, or via image_style.
        elseif (!isset($item_attributes[$key])) {
          $item_attributes[$key] = $item->{$key};
        }
      }
    }

    // Picture integration.
    if ($picture) {
      if ($mappings = picture_mapping_load($settings['responsive_image_style'])) {
        $settings['picture'] = picture_get_mapping_breakpoints($mappings, $settings['image_style']);
        $attributes['class'][] = 'media--picture';

        $image = [
          '#theme' => 'picture',
          '#uri' => $settings['uri'],
          '#breakpoints' => $settings['picture'],
          '#attributes' => $item_attributes,
        ];
      }
    }
    else {
      // If the image is lazyloaded.
      if ($settings['lazy']) {
        // Supports both lazyloaded or regular image.
        $item_attributes['class'][] = 'media__element';
        if ($settings['lazy']) {
          $item_attributes['src'] = Blazy::PLACEHOLDER;
          $attributes['class'][] = 'media--loading';
        }

        // Attach data attributes to either IMG tag, or DIV container.
        if (!$settings['background'] || empty($settings['blazy'])) {
          Blazy::buildBreakpointAttributes($item_attributes, $settings, $item);
        }

        // Supports Slick and Blazy CSS background lazyloading on DIV element.
        if ($settings['background']) {
          Blazy::buildBreakpointAttributes($attributes, $settings, $item);
          $attributes['class'][] = 'media--background';
        }

        // Multi-breakpoint aspect ratio only applies if lazyloaded.
        if (!empty($settings['blazy_data']['dimensions'])) {
          $attributes['data-dimensions'] = drupal_json_encode($settings['blazy_data']['dimensions']);
        }

        // Do not pass to theme_image() as D7 doesn't support data URI, yet.
        $image['#markup'] = '<img' . drupal_attributes($item_attributes) . ' />';
      }
      else {
        // Supports non-lazyloaded images such as for optional Slick lazyload.
        // As we pass image_url, not URI, no worries about theme_image_style().
        $image = [
          '#theme' => 'image',
          '#item' => (array) $item,
          '#path' => $settings['image_url'],
        ];
        foreach ($item_attributes as $key => $attribute) {
          $key = $key == 'typeof' ? 'attributes' : $key;
          $image['#' . $key] = $attribute;
        }
      }
    }

    // Provides extra attributes as needed.
    foreach (['caption', 'media', 'wrapper'] as $key) {
      if (!empty($settings[$key . '_attributes'])) {
        $element["#$key" . '_attributes'] = $settings[$key . '_attributes'];
        unset($settings[$key . '_attributes']);
      }
    }

    // Provides data for the renderable elements.
    $element['#image']      = $image;
    $element['#item']       = $item;
    $element['#captions']   = empty($build['captions']) ? [] : ['inline' => $build['captions']];
    $element['#attributes'] = $attributes;
    $element['#settings']   = $settings;
  }

  /**
   * Cleans up not so empty breakpoints.
   *
   * @param array $settings
   *   The settings being modified.
   */
  public function cleanUpBreakpoints(array &$settings) {
    if (!empty($settings['breakpoints'])) {
      foreach ($settings['breakpoints'] as $key => &$breakpoint) {
        $breakpoint = array_filter($breakpoint);

        if (empty($breakpoint['width']) && empty($breakpoint['image_style'])) {
          unset($settings['breakpoints'][$key]);
        }
      }
    }

    // Identify that Blazy can be activated only by breakpoints.
    if (empty($settings['blazy'])) {
      $settings['blazy'] = !empty($settings['breakpoints']);
    }
  }

  /**
   * Checks if an image style contains crop effect.
   *
   * @param string $image_style
   *   The image style to check for.
   *
   * @return bool
   *   TRUE if the image style contains crop effect, else FALSE.
   */
  public function isCrop($image_style) {
    if (!isset($this->isCrop[$image_style])) {
      $this->isCrop[$image_style] = FALSE;
      foreach (image_styles()[$image_style]['effects'] as $effect) {
        if (strpos($effect['name'], 'crop') !== FALSE) {
          $this->isCrop[$image_style] = TRUE;
          break;
        }
      }
    }
    return $this->isCrop[$image_style];
  }

  /**
   * Checks for Blazy formatter such as from within a Views style plugin.
   *
   * Ensures the settings traverse up to the container where Blazy is clueless.
   * The supported plugins can add [data-blazy] attribute into its container
   * containing $settings['blazy_data'] converted into [data-blazy] JSON.
   *
   * @param array $settings
   *   The settings being modified.
   * @param array $item
   *   The item containing settings or item keys, not image item.
   */
  public function isBlazy(array &$settings, array $item = []) {
    if (!$this->isBlazy) {
      // Retrieves Blazy formatter related settings from within Views style.
      $content = !empty($settings['item_id']) && $item && isset($item[$settings['item_id']]) ? $item[$settings['item_id']] : $item;

      // Blazy formatter within Views fields by supported modules.
      if (isset($item['settings'])) {
        // Prevents edge case with unexpected flattened Views results which is
        // normally triggered by checking "Use field template" option.
        $blazy = is_array($content) && isset($content['#build']['settings']) ? $content['#build']['settings'] : [];

        // Allows breakpoints overrides such as GridStack multi-styled images.
        if (empty($settings['breakpoints']) && isset($blazy['breakpoints'])) {
          $settings['breakpoints'] = $blazy['breakpoints'];
        }

        $cherries = [
          'blazy',
          'box_style',
          'image_style',
          'lazy',
          'media_switch',
          'ratio',
          'uri',
        ];

        foreach ($cherries as $key) {
          $fallback = isset($settings[$key]) ? $settings[$key] : '';
          $settings[$key] = isset($blazy[$key]) && empty($fallback) ? $blazy[$key] : $fallback;
        }
      }

      // Provides data for the [data-blazy] attribute at the containing element.
      $this->cleanUpBreakpoints($settings);
      if (!empty($settings['breakpoints'])) {
        $image = isset($item['item']) ? $item['item'] : NULL;
        $this->buildDataBlazy($settings, $image);
      }
      $this->isBlazy = TRUE;
    }
  }

  /**
   * Builds breakpoints suitable for top-level [data-blazy] wrapper attributes.
   *
   * The hustle is because we need to define dimensions once, if applicable, and
   * let all images inherit. Each breakpoint image may be cropped, or scaled
   * without a crop. To set dimensions once requires all breakpoint images
   * uniformly cropped. But that is not always the case.
   *
   * @param array $settings
   *   The settings being modified.
   * @param object|mixed $item
   *   The \Drupal\image\Plugin\Field\FieldType\ImageItem item, or array when
   *   dealing with Video Embed Field.
   *
   * @todo: Refine this like everything else.
   */
  public function buildDataBlazy(array &$settings, $item = NULL) {
    // Early opt-out if blazy_data has already been defined.
    // Blazy doesn't always deal with image directly.
    if (!empty($settings['blazy_data'])) {
      return;
    }

    if (empty($settings['original_width'])) {
      $settings['original_width'] = isset($item->width) ? $item->width : NULL;
      $settings['original_height'] = isset($item->height) ? $item->height : NULL;
    }

    $json = $sources = [];
    $end = end($settings['breakpoints']);
    foreach ($settings['breakpoints'] as $key => $breakpoint) {
      if (empty($breakpoint['image_style']) || empty($breakpoint['width'])) {
        continue;
      }

      if ($width = Blazy::widthFromDescriptors($breakpoint['width']) && !empty($settings['ratio'])) {
        // If contains crop, sets dimension once, and let all images inherit.
        if ($this->isCrop($breakpoint['image_style'])) {
          $dimensions = Blazy::transformDimensions($breakpoint['image_style'], $item);
          $padding = round((($dimensions['height'] / $dimensions['width']) * 100), 2);

          $json['dimensions'][$width] = $padding;

          // Only set padding-bottom for the last breakpoint to avoid FOUC.
          if ($end['width'] == $breakpoint['width']) {
            $settings['padding_bottom'] = $padding;
          }
        }

        // If BG, provide [data-src-BREAKPOINT].
        if (!empty($settings['background'])) {
          $sources[] = ['width' => (int) $width, 'src' => 'data-src-' . $key];
        }
      }
    }

    // As of Blazy v1.6.0 applied to BG only.
    if ($sources) {
      $json['breakpoints'] = $sources;
    }

    // @todo: A more efficient way not to do this in the first place.
    // ATM, this is okay as this method is run once on the top-level container.
    if (isset($json['dimensions']) && (count($settings['breakpoints']) != count($json['dimensions']))) {
      unset($json['dimensions'], $settings['padding_bottom']);
    }

    // Supported modules can add blazy_data as [data-blazy] to the container.
    // This also informs individual images to not work with dimensions any more
    // if the image style contains 'crop'.
    if ($json) {
      $settings['blazy_data'] = $json;
    }

    // Identify that Blazy can be activated only by breakpoints.
    $settings['blazy'] = TRUE;
  }

}
